﻿using System.Drawing;
using System.Linq;
using System.Threading;

using MindFusion;
using MindFusion.Diagramming;
using MindFusion.Diagramming.WinForms;

using Action = System.Action;


namespace Animation
{
	class LinkAnimator
	{
		public LinkAnimator(DiagramLink link, DiagramView view)
		{
			this.link = link;
			this.view = view;
		}

		public void AnimatePath(
			int animationLength,
			params PointF[] points)
		{
			if (points.Length < 2)
				return;

			// total length of specified path
			float pathLen = Enumerable.
				Range(0, points.Length - 1).
				Aggregate(0f, (len, index) =>
					Utilities.Distance(points[index], points[index + 1]));

			// prepare the link
			link.ControlPoints.Clear();
			link.ControlPoints.Add(points[0]);
			link.ControlPoints.Add(points[0]);

			// animate from a thread
			var animationThread = new Thread(
				() =>
         		{
					for (int s = 0; s < points.Length - 1; s++)
					{
						// how much to grow the segment for each animation frame
						var segment = new Vector(points[s], points[s + 1]);
						float segmentTime = animationLength * segment.Length / pathLen;
						var increment = segment * FrameLen / segmentTime;

						// frames for current segment
						for (int f = 0; f < segmentTime / FrameLen; f++)
						{
							Thread.Sleep(FrameLen);
							link.EndPoint = link.EndPoint + increment;
							view.Invoke(new Action(
								() => link.UpdateFromPoints()));
						}

						// set exact position in case animation hasn't reached it due to rounding
						if (link.EndPoint != points[s + 1])
						{
							link.EndPoint = points[s + 1];
							view.Invoke(new Action(
						       	() => link.UpdateFromPoints()));
						}

						// add next point
						if (s < points.Length - 2)
							link.ControlPoints.Add(points[s + 1]);
					}
         		});
			animationThread.Start();
		}

		DiagramLink link;
		DiagramView view;

		const int FrameLen = 1000 / 30; // in milliseconds
	}
}